@@ -4,6 +4,7 @@ use strict;
 package Data::ICal::Entry;
 use base qw/Class::Accessor/;
 use Data::ICal::Property;
+use Sys::Hostname qw();         # For unique UIDs for entries
 use Carp;
 
 use constant CRLF => "\x0d\x0a";
@@ -22,8 +23,8 @@ Data::ICal::Entry - Represents an entry in an iCalendar file
 
     $calendar->add_entry($vtodo);
 
-    $event->add_entry($alarm); 
-    $event->add_entries($alarm1, ...); 
+    $event->add_entry($alarm);
+    $event->add_entries($alarm1, ...);
 
     # or all in one go
     my $vtodo = Data::ICal::Entry::Todo->new( \%props, \@entries );
@@ -62,8 +63,8 @@ sub new {
     $self->set( properties => {} );
     $self->set( entries    => [] );
     for (@_) {
-	ref $_ eq "HASH"  and $self->add_properties( %$_ );
-	ref $_ eq "ARRAY" and $self->add_entries( @$_ );
+        ref $_ eq "HASH"  and $self->add_properties( %$_ );
+        ref $_ eq "ARRAY" and $self->add_entries( @$_ );
     }
     return $self;
 }
@@ -86,6 +87,7 @@ versions of this module before 0.16.
 
 =cut
 
+my $uid = 0;
 sub as_string {
     my $self = shift;
     my %args = (
@@ -94,11 +96,22 @@ sub as_string {
     );
     my $output = $self->header(%args);
 
-    for my $name (
+    my @mandatory = (
         $self->mandatory_unique_properties,
-        $self->mandatory_repeatable_properties
-        )
-    {
+        $self->mandatory_repeatable_properties,
+    );
+
+    if (grep {$_ eq "uid"} @mandatory and !defined $self->properties->{uid}
+            and $self->auto_uid) {
+      # Per the RFC, create a "persistent, globally unique" UID for this
+      # event; "persistent" in this context does not mean consistent
+      # across time, but rather "unique across all time"
+      $self->add_property(
+          uid => time() . '-' .$$ . '-' . $uid++ . '@' . Sys::Hostname::hostname()
+      );
+    }
+
+    for my $name ( @mandatory ) {
         carp "Mandatory property for " . ( ref $self ) . " missing: $name"
             unless $self->properties->{$name}
                 and @{ $self->properties->{$name} };
@@ -145,6 +158,8 @@ sub add_entry {
     push @{ $self->{entries} }, $entry;
 
     $entry->vcal10( $self->vcal10 );
+    $entry->rfc_strict( $self->rfc_strict );
+    $entry->auto_uid( $self->auto_uid );
 
     return $self;
 }
@@ -276,7 +291,7 @@ sub add_properties {
     return $self;
 }
 
-=head2 mandatory_unique_properties 
+=head2 mandatory_unique_properties
 
 Subclasses should override this method (which returns an empty list by
 default) to provide a list of lower case strings identifying the
@@ -287,7 +302,7 @@ type.
 
 sub mandatory_unique_properties { () }
 
-=head2 mandatory_repeatable_properties 
+=head2 mandatory_repeatable_properties
 
 Subclasses should override this method (which returns an empty list by
 default) to provide a list of lower case strings identifying the
@@ -298,7 +313,7 @@ type.
 
 sub mandatory_repeatable_properties { () }
 
-=head2 optional_unique_properties 
+=head2 optional_unique_properties
 
 Subclasses should override this method (which returns an empty list by
 default) to provide a list of lower case strings identifying the
@@ -418,6 +433,30 @@ same value as their parents.
 
 __PACKAGE__->mk_accessors('vcal10');
 
+=head2 rfc_strict [$bool]
+
+Gets or sets a boolean saying whether this entry will complain about
+missing UIDs as per RFC2446. Defaults to false, for backwards
+compatibility.  Generally, you can just set this on your main
+L<Data::ICal> object when you construct it; C<add_entry> automatically
+makes sure that sub-entries end up with the same value as their parents.
+
+=cut
+
+__PACKAGE__->mk_accessors('rfc_strict');
+
+=head2 auto_uid [$bool]
+
+Gets or sets a boolean saying whether this entry should automatically
+generate its own persistently unique UIDs.  Defaults to false.
+Generally, you can just set this on your main L<Data::ICal> object when
+you construct it; C<add_entry> automatically makes sure that sub-entries
+end up with the same value as their parents.
+
+=cut
+
+__PACKAGE__->mk_accessors('auto_uid');
+
 =head2 header
 
 Returns the header line for the entry (including trailing newline).
@@ -464,7 +503,7 @@ my %_generic = (
 Translate a L<Text::vFile::asData> sub object into the appropriate
 L<Data::iCal::Event> subtype.
 
-=cut 
+=cut
 
 # TODO: this is currently recursive which could blow the stack -
 #       it might be worth refactoring to make it sequential
@@ -511,6 +550,8 @@ my %_action_map = (
     DISPLAY   => 'Display',
     EMAIL     => 'Email',
     PROCEDURE => 'Procedure',
+    NONE      => 'None',
+    URI       => 'URI',
 );
 
 # alarms have actions
@@ -522,7 +563,8 @@ sub _parse_valarm {
     die "Can't parse VALARM with action $action"
         unless exists $_action_map{$action};
 
-    my $alarm_class = "Data::ICal::Entry::Alarm::" . $_action_map{$action};
+    $action = $_action_map{$action};
+    my $alarm_class = "Data::ICal::Entry::Alarm::$action";
     eval "require $alarm_class";
     die "Failed to require $alarm_class : $@" if $@;
 
@@ -554,8 +596,8 @@ sub _parse_generic_event {
     my ( $parent, $entry, $object ) = @_;
 
     my $p = $object->{properties};
-    while ( my ( $key, $val ) = each(%$p) ) {
-        foreach my $occurence (@$val) {
+    for my $key ( sort keys %$p ) {
+        foreach my $occurence (@{ $p->{$key} }) {
             my $prop;
 
             # Unescapes, but only in v2, and not if it's explicitly not TEXT
@@ -584,40 +626,15 @@ sub _parse_generic_event {
 
 =head1 AUTHOR
 
-Jesse Vincent C<< <jesse@bestpractical.com> >> with David Glasser,
-Simon Wistow, and Alex Vandiver
+Best Practical Solutions, LLC E<lt>modules@bestpractical.comE<gt>
 
 =head1 LICENCE AND COPYRIGHT
 
-Copyright (c) 2005 - 2009, Best Practical Solutions, LLC.  All rights reserved.
+Copyright (c) 2005 - 2019, Best Practical Solutions, LLC.  All rights reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself. See L<perlartistic>.
 
-
-=head1 DISCLAIMER OF WARRANTY
-
-BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
-FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
-OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
-PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
-EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
-ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
-YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
-NECESSARY SERVICING, REPAIR, OR CORRECTION.
-
-IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
-WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
-REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
-LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
-OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
-THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
-RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
-FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
-SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
-SUCH DAMAGES.
-
 =cut
 
 1;
