@@ -15,13 +15,13 @@ Data::ICal::Entry::Event - Represents an event in an iCalendar file
     $vevent->add_properties(
         summary => "my party",
         description => "I'll cry if I want to",
-	# Dat*e*::ICal is not a typo here
+        # Dat*e*::ICal is not a typo here
         dtstart   => Date::ICal->new( epoch => time )->ical,
     );
 
     $calendar->add_entry($vevent);
 
-    $vevent->add_entry($alarm); 
+    $vevent->add_entry($alarm);
 
 =head1 DESCRIPTION
 
@@ -42,15 +42,27 @@ Returns C<VEVENT>, its iCalendar entry name.
 
 sub ical_entry_type {'VEVENT'}
 
+=head2 mandatory_unique_properties
+
+The C<uid> property is mandatory if C<rfc_strict> was passed to
+L<Data::ICal/new>.
+
+=cut
+
+sub mandatory_unique_properties {
+    my $self = shift;
+    return $self->rfc_strict ? ("uid") : ()
+}
+
 =head2 optional_unique_properties
 
 According to the iCalendar standard, the following properties may be
 specified at most one time for an event:
 
-	class  created  description  dtstart  geo 
-	last-modified  location  organizer  priority 
-	dtstamp  sequence  status  summary  transp 
-	uid  url  recurrence-id 
+        class  created  description  dtstart  geo
+        last-modified  location  organizer  priority
+        dtstamp  sequence  status  summary  transp
+        uid  url  recurrence-id
 
 In addition, C<dtend> and C<duration> may be specified at most once
 each, but not both in the same entry (though this restriction is not
@@ -67,23 +79,25 @@ Or if C<< vcal10 => 1 >>:
 
 sub optional_unique_properties {
     my $self = shift;
+    my @ret = $self->rfc_strict ? () : ("uid");
     if (not $self->vcal10) {
-        qw(
+        push @ret, qw(
             class  created  description  dtstart  geo
             last-modified  location  organizer  priority
             dtstamp  sequence  status  summary  transp
-            uid  url  recurrence-id
+            url  recurrence-id
 
             dtend duration
         );
     } else {
-        qw(
+        push @ret, qw(
             class dcreated completed description dtstart dtend
             last-modified location rnum priority
             sequence status summary transp
-            url uid
+            url
         );
     }
+    return @ret;
 }
 
 =head2 optional_repeatable_properties
@@ -91,9 +105,9 @@ sub optional_unique_properties {
 According to the iCalendar standard, the following properties may be
 specified any number of times for an event:
 
-	attach  attendee  categories  comment 
-	contact  exdate  exrule  request-status  related-to 
-	resources  rdate  rrule  
+        attach  attendee  categories  comment
+        contact  exdate  exrule  request-status  related-to
+        resources  rdate  rrule
 
 Or if C<< vcal10 => 1 >>:
 
@@ -135,40 +149,15 @@ defines.
 
 =head1 AUTHOR
 
-Jesse Vincent C<< <jesse@bestpractical.com> >> with David Glasser,
-Simon Wistow, and Alex Vandiver
+Best Practical Solutions, LLC E<lt>modules@bestpractical.comE<gt>
 
 =head1 LICENCE AND COPYRIGHT
 
-Copyright (c) 2005 - 2009, Best Practical Solutions, LLC.  All rights reserved.
+Copyright (c) 2005 - 2019, Best Practical Solutions, LLC.  All rights reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself. See L<perlartistic>.
 
-
-=head1 DISCLAIMER OF WARRANTY
-
-BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
-FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
-OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
-PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
-EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
-ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
-YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
-NECESSARY SERVICING, REPAIR, OR CORRECTION.
-
-IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
-WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
-REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
-LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
-OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
-THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
-RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
-FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
-SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
-SUCH DAMAGES.
-
 =cut
 
 1;
